@@ -1866,6 +1866,9 @@ set_root_info (GVfsBackendAfp *afp_backend, GFileInfo *info)
   icon = g_vfs_backend_get_icon (G_VFS_BACKEND (afp_backend));
   if (icon != NULL)
     g_file_info_set_icon (info, icon);
+  icon = g_vfs_backend_get_symbolic_icon (G_VFS_BACKEND (afp_backend));
+  if (icon != NULL)
+    g_file_info_set_symbolic_icon (info, icon);
 }
 
 static void
@@ -2055,18 +2058,19 @@ do_mount (GVfsBackend *backend,
   
   if (afp_backend->user)
     /* Translators: first %s is volumename, second username and third servername */ 
-    display_name = g_strdup_printf (_("AFP volume %s for %s on %s"), 
+    display_name = g_strdup_printf (_("%s for %s on %s"),
                                     afp_backend->volume_name, afp_backend->user,
                                     server_name);
   else
     /* Translators: first %s is volumename and second servername */
-    display_name = g_strdup_printf (_("AFP volume %s on %s"),
+    display_name = g_strdup_printf (_("%s on %s"),
                                     afp_backend->volume_name, server_name);
   
   g_vfs_backend_set_display_name (backend, display_name);
   g_free (display_name);
 
   g_vfs_backend_set_icon_name (backend, "folder-remote-afp");
+  g_vfs_backend_set_symbolic_icon_name (backend, "folder-remote-symbolic");
   g_vfs_backend_set_user_visible (backend, TRUE);
 
   g_vfs_job_succeeded (G_VFS_JOB (job));
