@@ -46,6 +46,11 @@ enum {
   PROP_0
 };
 
+enum {
+  SHUTDOWN,
+  LAST_SIGNAL
+};
+
 typedef struct {
   char *obj_path;
   GVfsRegisterPathCallback callback;
@@ -86,6 +91,8 @@ typedef struct {
   GDBusConnection *conn;
 } NewConnectionData;
 
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static void              g_vfs_daemon_get_property (GObject        *object,
 						    guint           prop_id,
 						    GValue         *value,
@@ -173,6 +180,15 @@ g_vfs_daemon_class_init (GVfsDaemonClass *klass)
   gobject_class->finalize = g_vfs_daemon_finalize;
   gobject_class->set_property = g_vfs_daemon_set_property;
   gobject_class->get_property = g_vfs_daemon_get_property;
+
+  signals[SHUTDOWN] =
+    g_signal_new ("shutdown",
+		  G_TYPE_FROM_CLASS (gobject_class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (GVfsDaemonClass, shutdown),
+		  NULL, NULL,
+		  g_cclosure_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 }
 
 static void
@@ -344,9 +360,9 @@ g_vfs_daemon_set_max_threads (GVfsDaemon                    *daemon,
 }
 
 static gboolean
-exit_at_idle (gpointer data)
+exit_at_idle (GVfsDaemon *daemon)
 {
-  exit (0);
+  g_signal_emit (daemon, signals[SHUTDOWN], 0);
   return FALSE;
 }
 
@@ -364,7 +380,7 @@ static void
 daemon_schedule_exit (GVfsDaemon *daemon)
 {
   if (daemon->exit_tag == 0)
-    daemon->exit_tag = g_timeout_add_seconds (1, exit_at_idle, daemon);
+    daemon->exit_tag = g_timeout_add_seconds (1, (GSourceFunc)exit_at_idle, daemon);
 }
 
 static void
@@ -608,35 +624,19 @@ peer_connection_closed (GDBusConnection *connection,
   GVfsDaemon *daemon = G_VFS_DAEMON (user_data);
   GList *l;
   GVfsDBusDaemon *daemon_skeleton;
-  GVfsJob *job_to_cancel;
 
-  do
+  g_mutex_lock (&daemon->lock);
+  for (l = daemon->jobs; l != NULL; l = l->next)
     {
-      job_to_cancel = NULL;
-
-      g_mutex_lock (&daemon->lock);
-      for (l = daemon->jobs; l != NULL; l = l->next)
-        {
-          GVfsJob *job = G_VFS_JOB (l->data);
-
-          if (G_VFS_IS_JOB_DBUS (job) &&
-              !g_vfs_job_is_cancelled (job) &&
-              G_VFS_JOB_DBUS (job)->invocation &&
-              g_dbus_method_invocation_get_connection (G_VFS_JOB_DBUS (job)->invocation) == connection)
-            {
-              job_to_cancel = g_object_ref (job);
-              break;
-            }
-        }
-      g_mutex_unlock (&daemon->lock);
-
-      if (job_to_cancel)
-        {
-          g_vfs_job_cancel (job_to_cancel);
-          g_object_unref (job_to_cancel);
-        }
+      GVfsJob *job = G_VFS_JOB (l->data);
+      
+      if (G_VFS_IS_JOB_DBUS (job) &&
+          G_VFS_JOB_DBUS (job)->invocation &&
+          g_dbus_method_invocation_get_connection (G_VFS_JOB_DBUS (job)->invocation) == connection)
+        g_vfs_job_cancel (job);
     }
-  while (job_to_cancel != NULL);
+  g_mutex_unlock (&daemon->lock);
+
 
   daemon_skeleton = g_object_get_data (G_OBJECT (connection), "daemon_skeleton");
   /* daemon_skeleton should be always valid in this case */
@@ -1045,11 +1045,13 @@ g_vfs_daemon_run_job_in_thread (GVfsDaemon *daemon,
 }
 
 void
-g_vfs_daemon_close_active_channels (GVfsDaemon *daemon)
+g_vfs_daemon_close_active_channels (GVfsDaemon *daemon,
+				    GVfsBackend *backend)
 {
   GList *l;
 
    for (l = daemon->job_sources; l != NULL; l = l->next)
-      if (G_VFS_IS_CHANNEL (l->data))
+      if (G_VFS_IS_CHANNEL (l->data) &&
+          g_vfs_channel_get_backend (G_VFS_CHANNEL (l->data)) == backend)
         g_vfs_channel_force_close (G_VFS_CHANNEL (l->data));
 }
